# PyRIC v 1.6 root Distribution directory
# Do not import from this directory i.e from PyRIC import foobar
# To install
#   use pip ('sudo pip install PyRIC')
# or download latest tarbal and untar.
# excute either sudo python setup.py or rund from the PyRIC/pyric directory
